@@ -1,9 +1,10 @@
 /*
  * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 2 as published
- * by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -210,7 +211,7 @@ null_device_read_label(Device * dself) {
 
     device_set_error(dself,
 	stralloc(_("Can't open NULL device for reading or appending.")),
-	DEVICE_STATUS_DEVICE_ERROR);
+	DEVICE_STATUS_VOLUME_UNLABELED | DEVICE_STATUS_VOLUME_ERROR);
     return FALSE;
 }
 
@@ -248,7 +249,7 @@ null_device_start (Device * pself, DeviceAccessMode mode,
     } else {
 	device_set_error(pself,
 	    stralloc(_("Can't open NULL device for reading or appending.")),
-	    DEVICE_STATUS_DEVICE_ERROR);
+	    DEVICE_STATUS_VOLUME_UNLABELED | DEVICE_STATUS_VOLUME_ERROR);
         return FALSE;
     }
 }
