@@ -1,9 +1,10 @@
 /*
  * Copyright (c) 2008-2012 Zmanda, Inc.  All Rights Reserved.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 2 as published
- * by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -31,7 +32,8 @@ typedef enum {
    S3_API_S3,
    S3_API_SWIFT_1,
    S3_API_SWIFT_2,
-   S3_API_OAUTH2
+   S3_API_OAUTH2,
+   S3_API_CASTOR
 } S3_api;
 
 /* An opaque handle.  S3Handles should only be accessed from a single
@@ -262,7 +264,9 @@ s3_open(const char * access_key, const char *secret_key,
 	const char *client_id,
 	const char *client_secret,
 	const char *refresh_token,
-	const gboolean reuse_connection);
+	const gboolean reuse_connection,
+        const char *reps,
+        const char *reps_bucket);
 
 /* latest step of setting up the S3Handle.
  *
