@@ -1,9 +1,10 @@
 /*
  * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 2 as published
- * by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -605,11 +606,12 @@ tape_device_set_read_block_size_fn(Device *p_self, DevicePropertyBase *base G_GN
 
     if (read_block_size != 0 &&
 	    ((gsize)read_block_size < p_self->block_size ||
-	     (gsize)read_block_size > p_self->max_block_size))
+	     (gsize)read_block_size > p_self->max_block_size)) {
 	device_set_error(p_self,
 	    g_strdup_printf("Error setting READ-BLOCk-SIZE property to '%u', it must be between %zu and %zu", read_block_size, p_self->block_size, p_self->max_block_size),
 	    DEVICE_STATUS_DEVICE_ERROR);
 	return FALSE;
+    }
 
     self->private->read_block_size = read_block_size;
 
