@@ -2,9 +2,10 @@
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 2009-2012 Zmanda, Inc.  All Rights Reserved.
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License version 2 as published
- * by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -565,7 +566,7 @@ part_done:
     /* time runs backward on some test boxes, so make sure this is positive */
     if (msg->duration < 0) msg->duration = 0;
 
-    if (msg->successful)
+    if (msg->successful && msg->size > 0)
 	self->partnum++;
     self->no_more_parts = msg->eof || (!msg->successful && !self->expect_cache_inform);
 
