@@ -1,8 +1,9 @@
 # Copyright (c) 2009-2012 Zmanda, Inc.  All Rights Reserved.
 #
-# This program is free software; you can redistribute it and/or modify it
-# under the terms of the GNU General Public License version 2 as published
-# by the Free Software Foundation.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License
+# as published by the Free Software Foundation; either version 2
+# of the License, or (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful, but
 # WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -91,6 +92,9 @@ my ($fc14, $fc15);
     if ($uname =~ /\.fc16\./) { #like fc15
 	$fc15 = 1;
     }
+    if ($uname =~ /\.fc17\./) { #like fc15
+	$fc15 = 1;
+    }
 }
 
 # see if the default for --wildcards during inclusion has been changed
