@@ -1,8 +1,9 @@
 # Copyright (c) 2010-2012 Zmanda, Inc.  All Rights Reserved.
 #
-# This library is free software; you can redistribute it and/or modify it
-# under the terms of the GNU Lesser General Public License version 2.1 as
-# published by the Free Software Foundation.
+# This library is free software; you can redistribute it and/or
+# modify it under the terms of the GNU Lesser General Public
+#* License as published by the Free Software Foundation; either
+# version 2.1 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful, but
 # WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -303,7 +304,7 @@ sub quit {
     my $finished_cb = $params{'finished_cb'};
 
     confess "Cannot quit a Clerk while a transfer is in progress"
-	if $self->{'xfer_state'};
+	if $self->{'xfer_state'} and $self->{'xfer_state'}->{'xfer'};
 
     my $steps = define_steps 
 	cb_ref => \$finished_cb,
