@@ -1,8 +1,9 @@
 # Copyright (c) 2010-2012 Zmanda, Inc.  All Rights Reserved.
 #
-# This program is free software; you can redistribute it and/or modify it
-# under the terms of the GNU General Public License version 2 as published
-# by the Free Software Foundation.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License
+# as published by the Free Software Foundation; either version 2
+# of the License, or (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful, but
 # WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -575,6 +576,7 @@ sub volume_is_labelable {
 	if (!$autolabel->{'other_config'}) {
 	    $self->_user_msg(slot_result  => 1,
 			     label        => $label,
+			     labelstr     => $self->{'labelstr'},
 			     does_not_match_labelstr => 1,
 			     slot         => $slot);
 	    return 0;
