@@ -1,8 +1,9 @@
 # Copyright (c) 2009-2012 Zmanda Inc.  All Rights Reserved.
 #
-# This program is free software; you can redistribute it and/or modify it
-# under the terms of the GNU General Public License version 2 as published
-# by the Free Software Foundation.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License
+# as published by the Free Software Foundation; either version 2
+# of the License, or (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful, but
 # WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -114,7 +115,7 @@ sub start {
 	tx_fh => *STDOUT,
 	message_cb => $message_cb,
 	message_obj => $self,
-	debug => $Amanda::Config::debug_taper?'driver/taper':'',
+	debug => $Amanda::Config::debug_taper?'taper/driver':'',
     );
 
     my $changer = Amanda::Changer->new(undef, tapelist => $self->{'tapelist'});
