@@ -1,6 +1,7 @@
 /*
  * Amanda, The Advanced Maryland Automatic Network Disk Archiver
  * Copyright (c) 1991-2000 University of Maryland at College Park
+ * Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
  * All Rights Reserved.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -71,7 +72,7 @@ int test_server_pgm(FILE *outf, char *dir, char *pgm, int suid, uid_t dumpuid);
 void
 usage(void)
 {
-    g_printf(_("Usage: amcheck [--version] [-am] [-w] [-sclt] [-M <address>] [--client-verbose] [-o configoption]* <conf> [host [disk]* ]*\n"));
+    g_printf(_("Usage: amcheck [--version] [-am] [-w] [-sclt] [-M <address>] [--client-verbose] [--exact_match] [-o configoption]* <conf> [host [disk]* ]*\n"));
     exit(1);
     /*NOTREACHED*/
 }
@@ -82,9 +83,11 @@ static char *displayunit;
 static long int unitdivisor;
 
 static int client_verbose = FALSE;
+static gboolean exact_match = FALSE;
 static struct option long_options[] = {
     {"client-verbose", 0, NULL,  1},
     {"version"       , 0, NULL,  2},
+    {"exact-match"   , 0, NULL,  3},
     {NULL, 0, NULL, 0}
 };
 
@@ -174,6 +177,8 @@ main(
 	case 2:		printf("amcheck-%s\n", VERSION);
 			return(0);
 			break;
+	case 3:		exact_match = TRUE;
+			break;
 	case 'M':	if (mailto) {
 			    g_printf(_("Multiple -M options\n"));
 			    exit(1);
@@ -278,7 +283,7 @@ main(
 
     conf_ctimeout = (time_t)getconf_int(CNF_CTIMEOUT);
 
-    errstr = match_disklist(&origq, argc-1, argv+1);
+    errstr = match_disklist(&origq, exact_match, argc-1, argv+1);
     if (errstr) {
 	g_printf(_("%s"),errstr);
 	amfree(errstr);
