@@ -1,9 +1,10 @@
 #! @PERL@
 # Copyright (c) 2007-2012 Zmanda, Inc.  All Rights Reserved.
 # 
-# This program is free software; you can redistribute it and/or modify it
-# under the terms of the GNU General Public License version 2 as published 
-# by the Free Software Foundation.
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License
+# as published by the Free Software Foundation; either version 2
+# of the License, or (at your option) any later version.
 # 
 # This program is distributed in the hope that it will be useful, but
 # WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -518,7 +519,7 @@ sub main {
 	if ($all_success) {
 	    print "All images successfully validated\n";
 	} else {
-	    print "Some images failed to be correclty validated.\n";
+	    print "Some images failed to be correctly validated.\n";
 	    $exit_code = 1;
 	}
 
